"""
    Copyright 2006,2007 Riccardo Taormina (den3b@hotmail.com)
    
    This file is part of SmartChameleons.

    SmartChameleons is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    SmartChameleons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SmartChameleons; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import random

from Tkinter import *
from Panel import *
from Model import *
from OutputWindow import *
from ActionGroup import ActionGroup
import tkMessageBox
from InputGUI import InputGUI
from globalVars import gifdir, colors




class Observer(Tk):
    def __init__(self):
        Tk.__init__(self)

        self.title("Observer Environment")
        
        self.visible = True
        self.envDict = {}
        self.chamDicts = {}
        self.stopIt = False
        self.onlyOneColor = False
        self.speed = 1
        self.refreshFrequency = 1
        self.forceExit = False
        self.outputToFile = []
        self.protocol("WM_DELETE_WINDOW", self.handler)
        

    def buildObjects(self):
        #build the input GUI components
        self.withdraw()
        self.inputGUI = InputGUI(self)

        #check if it works
        self.colors=colors[0:self.envDict["nBreeds"]]
        self.cham_images = {}
        for color in self.colors:
            self.cham_images[color] = PhotoImage(file = gifdir + color + ".gif")

        #StringVar required fot the GUI Output window. CHECK THIS!!!!! seems awful
        self.breedNumbers = {}
        for color in self.colors:
            var = IntVar()
            var.set(self.chamDicts[color]["startNumber"])
            self.breedNumbers[color]=var
            
        self.time = StringVar()
        self.time.set('')
            

        self.panel = Panel(self)
        self.output = OutputWindow(self)
        self.deiconify()

        

    def create(self):
        self.maxX=self.envDict["worldXsize"]
        self.maxY=self.envDict["worldYsize"]
        self.n_cycles=self.envDict["nCycles"]
        self.seed = self.envDict["seed"]

        #initialize the randomizer
        random.seed(self.seed)

        #create the canvas showing the environment
        self.canvas = Canvas(self,background = "white",width=self.maxX*36,height=self.maxY*36)
        self.canvas.pack()

        #build the Model
        self.model = Model(self,self.envDict,self.chamDicts,self.colors)
        self.model.buildObjects()
        self.model.buildActions()
        self.refreshActionGroup.actions[0](self) # RefreshOutput this is only to store the initial number to disk


    def buildActions(self):
    

        self.refreshActionGroup = ActionGroup ("refresh")

        def refreshOutput(self):
            self.time.set(str(self.model.time)+"/"+str(self.n_cycles))
            temp = ''
            for color in self.colors:
                temp = temp + str(self.model.breedNumbers[color]) + ' '
            temp = temp + '\n'
            self.outputToFile.append(temp)
            
        def refreshEnvironment(self):
            if (self.model.time % self.refreshFrequency == 0) :
                for chameleon in self.model.chList:
                    if self.visible:
                        item = chameleon.graphItem
                        self.canvas.coords(item,chameleon.xPosition*36,chameleon.yPosition*36)                
                    
        
        def insertDelay(self):
            if self.speed < 10:
                self.after(500/self.speed)

        def update(self):
            if (self.model.time % self.refreshFrequency == 0):
                self.update()

        self.refreshActionGroup.actions = [refreshOutput,refreshEnvironment,insertDelay,update]

        self.actionGroupList = ["step", "refresh"]


    def go(self):

        [modelIsRunning, message] = self.checkUp()

        while modelIsRunning:
            for action in self.actionGroupList:
                if action == "step":
                    self.model.step()
                elif action == "refresh":
                    for refreshAction in self.refreshActionGroup.actions:
                        refreshAction(self)

            [modelIsRunning, message] = self.checkUp()

        if message != 'stopped by user':
            if message == "exit forced":
                self.destroy()
            else :
                tkMessageBox.showwarning("Message", message)
                self.storeOutput()
                self.destroy()


    def handler(self):
        self.panel.focus_set()
        self.withdraw()


    def quit(self):
        self.storeOutput()
        try :
            if self.model.time == 0 or self.model.time==self.n_cycles or self.stopIt or self.onlyOneColor:
                self.destroy()
            else :
                self.forceExit=True
        except AttributeError:
            self.destroy()


    def checkUp(self):
        flag = True
        message = ''
        if self.onlyOneColor :
            self.panel.go_b["state"]=DISABLED
            message = 'Chameleons stopped changing at cycle num. ' + str(self.model.time) + \
                      '; ' + self.model.chList[0].color + ' wins'
            flag = False

        elif self.model.time == self.n_cycles :
            self.panel.go_b["state"]=DISABLED
            message = str(self.n_cycles) + ' have passed. Neither color is the winner.'
            flag = False

        elif self.forceExit :
            message = 'exit forced'
            flag = False

        elif self.stopIt :
            message = 'stopped by user'
            flag = False

        else :
            message = "YES"

        return flag,message


    def storeOutput(self):
        i=0
        while 1:
            path='output'
            if i == 0:
                add = '.txt'
            else :
                add = str(i) + '.txt'
            path = path + add
            try :
                file_=open(path,'r')
                file_.close()
                i+=1
                continue
            except IOError:
                file_=open(path,'w')
                file_.writelines(self.outputToFile)
                file_.close
                break
            
            
            
            




        
        
